<?php

namespace Drupal\dkan_reference_fix\Reference;

use Contracts\FactoryInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\metastore\Reference\MetastoreUrlGenerator;
use Drupal\metastore\SchemaRetriever;
use Drupal\node\NodeInterface;
use GuzzleHttp\Client;
use Psr\Log\LoggerInterface;
use Symfony\Component\Mime\MimeTypeGuesserInterface;

class Referencer extends \Drupal\metastore\Reference\Referencer {

  protected $schemaRetriever;

  public function __construct(ConfigFactoryInterface $configService, FactoryInterface $storageFactory, MetastoreUrlGenerator $metastoreUrlGenerator, Client $httpClient, MimeTypeGuesserInterface $mimeTypeGuesser, LoggerInterface $loggerChannel, SchemaRetriever $schema_retriever)
  {
    parent::__construct($configService, $storageFactory, $metastoreUrlGenerator, $httpClient, $mimeTypeGuesser, $loggerChannel);
    $this->schemaRetriever = $schema_retriever;
  }

  public function reference($data)
  {
    if (!is_object($data)) {
      throw new \Exception('data must be an object.');
    }
    // Cycle through the dataset properties we seek to reference.
    foreach ($this->getPropertyList() as $property_id) {
      if (isset($data->{$property_id})) {
        $reference_property = $this->referenceProperty($property_id, $data->{$property_id});
        if ($reference_property) {
          $data->{$property_id} = $reference_property;
        }
        else {
          unset($data->{$property_id});
        }

        // Remove de-referenced info from metadata.
        unset($data->{'%Ref:' . $property_id});
      }
    }
    return $data;
  }

  protected function referenceSingle(string $property_id, $value)
  {
    $dataset_schema_ui = json_decode($this->schemaRetriever->retrieve('dataset.ui'));
    $title_property = $dataset_schema_ui->{$property_id}->{'ui:options'}->titleProperty ?? FALSE;
    if ($title_property) {
      $label_value = $value->{$title_property};
      if (!empty($label_value)) {
        $pub_nodes = \Drupal::entityTypeManager()->getStorage('node')->loadByProperties([
          'field_data_type' => $property_id,
          'json_label_property_value' => $label_value
        ]);
        $pub_node = reset($pub_nodes);
        if ($pub_node instanceof NodeInterface && $pub_node->hasField('field_json_metadata') && !$pub_node->get('field_json_metadata')->isEmpty()) {
          return $pub_node->uuid();
        }
        else {
          // No existing reference found, this means a new one. In that case we should delete properties, except for defaults and title
          $prop_schema = json_decode($this->schemaRetriever->retrieve($property_id));
          $new_value = new \stdClass();
          foreach ($prop_schema->properties->data->properties as $property_name => $property) {
            if (isset($property->default)) {
              $new_value->$property_name = $property->default;
            }
          }
          $new_value->$title_property = $value->$title_property;
          $value = $new_value;
        }
      }
      else {
        // empty value for title property means removing the reference !?
        return NULL;
      }
    }
    return parent::referenceSingle($property_id, $value);
  }
}
